## Requirements

In order to build the code that is supplied with this document, the follow is required:

 - The STM32L100C-DISCO board. For other boards, it is necessary to replace stm32l1_wrapper.c and the linker script stm32l15xxc.ld

 - The arm-none-eabi toolchain. The easiest way to obtain this is probably via [the ARM-embedded launchpad page][1].

 - libopencm3. This library can be obtained from [libopencm3.org][2].

 - The st-link utilities. These are used to flash and debug the STM32 board, and can be obtained from [the ST-link Github repository][3].

 - A way to interface with the board's USART controller. For this project, we used a Prolific PL2303 USB to TTL converter. These are widely available at low costs.

[1]: https://launchpad.net/gcc-arm-embedded
[2]: http://libopencm3.org
[3]: https://github.com/texane/stlink

## Building

There is a Makefile supplied with the source code in both the `sphincs_stm32l1` and the `xmss_stm32l1` directories. Make sure to point the OPENCM3DIR variable in this Makefile to the right location. This should be the location where one can find the `include` directory with libopencm3 header files inside.

Assuming that the arm-none-eabi toolchain was installed properly, it should now be possible to compile the code.

## Hostside

All code in this package assumes a host that can play a role in the communication by either providing a message or signature to verify, or simply receiving the output. The host typically runs the Python scripts included in the `hostside` directory. The hostside scripts assume that the board can be reached using stty over device `/dev/ttyUSB0`. This can be changed to a different serial device as needed.

## Connecting the board

When using a USB to TTL converter, there are typically five pins available. Connect them as follows:

- 3V3 -> 3V
- TXD -> PA3
- RXD -> PA2
- GDN -> GND

Note that the 3V connection is only necessary when not supplying power to the board through some other means, such as through the mini-USB interface used for flashing.

## Flashing

After building the code, one should have a `.bin` file that can be flashed onto the board. This can be done by calling e.g. `st-flash write verifier.bin 0x8000000`. The st-flash utility is part of the st-link project mentioned above.

## Running the code

In order to run the code, one should always start one of the applications on the hostside first. These programs wait for a 'request for data' from the board, in the form of a 1-byte message. This is used to trigger a series of DMA communications.

The `generic_receiver.py` script does not expect any input files and will just output the data that the board sends it. Use this to try the XMSS code, as this currently does not use any message input.

The `sphincs_signer_host.py` file expects a filename as second parameter, containing the message to be signed. Optionally, one can supply the options -u (to unscramble the HORST signature) and -m (to append the message itself to the signature, for easier verification).

The `sphincs_verifier_host.py` file expects the filename of a file containing a signature and the message (in that order).

## Troubleshooting

It may happen that the board stops responding and/or does not accept communication or new flashed code. This typically occurs when the flashing process was interrupted or the firmware checksum does not match after flashing (e.g. when the binary file has changed). A smaller manifestation of a problem with the board can be the fact that it will appear to be sending out erroneous bytes at the start of each communication.

At some point the boards might behave differently than one would expect, to a point where simply power-cycling the board does not help. In these cases, it is useful to be aware of a few trouble-shooting steps.

First, check if all the cables are attached properly. For the boards supported in this repository, connect TX to PA3, RX to PA2 and GND to GND. Power is typically supplied using the mini-USB connector that is also used to flash code onto the board.

If the code in this repository does not appear to work correctly after flashing it on to the board, try pressing the RST button (optionally followed by re-flashing).

If you cannot flash new code onto the board, but are instead confronted with WARN src/stlink-common.c: unknown chip id!, try shorting the BOOT0 and VDD pins and pressing RST. This selects the DFU bootloader. After that, optionally use st-flash erase before re-flashing the board.
