#!/usr/bin/env python3
import serial
import sys
import os
import argparse

dev = serial.Serial("/dev/ttyUSB0", 921600)

parser = argparse.ArgumentParser(description='Host-side script for SPHINCS verification')
parser.add_argument('msig', metavar='sig-message-file',
                    help='file that contains the signature+message')
args = parser.parse_args()

mlen = os.path.getsize(args.msig) - 41000

# block-lengths are based on SPHINCS-256 parameters
blengths = [32, 8, 64*32] + [32*11] * 32 + [2144+160] * 12

# current implementation expects a fixed 32-byte message
with open(args.msig, "rb") as sigm_file:
    sig = sigm_file.seek(41000)
    print("Waiting for request..", file=sys.stderr)
    dev.read(1)
    print("Writing message..", file=sys.stderr)
    dev.write(sigm_file.read(mlen))
    sigm_file.seek(0)
    for blen in blengths:
        x = dev.read(1)
        if (chr(x[0]) != 'x'):
            print("Got token 0x{:02X}..".format(x[0]), file=sys.stderr)
            print("Verificiation failed or interrupted!")
            break
        print("Writing {} bytes..".format(blen), file=sys.stderr)
        dev.write(sigm_file.read(blen))
    else:
        result = "failed!" if dev.read(1)[0] else "succeeded!"
        print("Verification", result, file=sys.stderr)

print("Any further output will now print to stderr:", file=sys.stderr)
while True:
    sys.stderr.buffer.write(dev.read(1))
    sys.stderr.flush()
