/*
version 20151006
Joost Rijneveld
Public domain.
*/

#include "stm32wrapper.h"
#include "params.h"
#include "xmss_fast.h"
#include <stdio.h>
#include <stdint.h>

int main(void)
{
  clock_setup();
  gpio_setup();
  usart_setup();
  dma_request_setup();
  dma_transmit_setup();

  uint32_t sk_address = 0x08080000;

  char buffer[32];
  int i, j;

  int m = XMSS_M;
  int n = XMSS_N;
  int h = XMSS_TREE_HEIGHT;
  int d = XMSSMT_LAYERS;
  int w = WOTS_W;
  int k = BDS_K;

  xmssmt_params p;
  xmssmt_params *params = &p;
  if (xmssmt_set_params(params, m, n, h, d, w, k)) {
    return 1;
  }

  unsigned int tree_h = h / d;

  // stack needs to be larger than regular (H-K-1), since we re-use for 'next'
  unsigned char stack[2*d * (tree_h + 1)*n];
  unsigned char stacklevels[2*d * (tree_h + 1)*n];
  unsigned char auth[2*d * tree_h*n];
  unsigned char keep[2*d * (tree_h >> 1)*n];
  treehash_inst treehash[2*d * (tree_h-k)];
  unsigned char th_nodes[2*d * (tree_h-k)*n];
  unsigned char retain[2*d * ((1 << k) - k - 1)*n];
  unsigned char wots_sigs[d * params->xmss_par.wots_par.keysize];
  bds_state states[2*d];

  for (i = 0; i < 2*d; i++) {
    for(j=0;j<h-k;j++) {
      treehash[i*(tree_h-k) + j].node = th_nodes + (i*(tree_h-k) + j) * n;
    }
    xmss_set_bds_state(states + i,
      stack + i*(tree_h + 1)*n, 0, stacklevels + i*(tree_h + 1),
      auth + i*tree_h*n,
      keep + i*(tree_h >> 1)*n,
      treehash + i*(tree_h-k),
      retain + i*((1 << k) - k - 1)*n,
      0
    );
  }

  // round (params->index_len+2*n+m) to nearest multiple of 4, then divide by 4
  int sk_length_words = (((params->index_len+2*n+m) + 4 - 1) & ~(4 - 1)) >> 2;

  unsigned char sk[4*sk_length_words];
  unsigned char pk[2*n];

  xmssmt_keypair(pk, sk, states, wots_sigs, params);

  // end of keygen, now write sk to EEPROM
  eeprom_program_words(sk_address, sk, sk_length_words); // 4 bytes per word

  char output[256];

  sprintf(output, "Public key: ");
  SendUARTStr(output);
  for (i = 0; i < 2*n; i++) {
    sprintf(output + i*2, "%02X", pk[i]);
  }
  SendUARTStr(output);

  sprintf(output, "Cost: %d", newcount);
  SendUARTStr(output);

  return 0;
}
