/*
version 20151006
Joost Rijneveld
Public domain.
*/

#include "stm32wrapper.h"
#include "params.h"
#include "xmss_fast.h"
#include <stdio.h>
#include <stdint.h>

#define MLEN 32
#define SIGNATURES 512

int main(void)
{
  clock_setup();
  gpio_setup();
  usart_setup();
  dma_request_setup();
  dma_transmit_setup();

  volatile unsigned int *DWT_CYCCNT = (unsigned int *)0xE0001004; //address of the register
  volatile unsigned int *DWT_CONTROL = (unsigned int *)0xE0001000; //address of the register
  volatile unsigned int *SCB_DEMCR = (unsigned int *)0xE000EDFC; //address of the register

  *SCB_DEMCR = *SCB_DEMCR | 0x01000000;
  *DWT_CYCCNT = 0; // reset the counter
  *DWT_CONTROL = *DWT_CONTROL | 1 ; // enable the counter

  unsigned char mi[MLEN];

  int m = XMSS_M;
  int n = XMSS_N;
  int h = XMSS_TREE_HEIGHT;
  int d = XMSSMT_LAYERS;
  int w = WOTS_W;
  int k = BDS_K;

  char output[256];
  sprintf(output, "Starting keygen/state setup, will treehash %dx for %d leafs.", d, 1 << (h/d));
  SendUARTStr(output);

  xmssmt_params p;
  xmssmt_params *params = &p;
  xmssmt_set_params(params, m, n, h, d, w, k);

  unsigned int tree_h = h / d;

  // stack needs to be larger than regular (H-K-1), since we re-use for 'next'
  unsigned char stack[2*d * (tree_h + 1)*n];
  unsigned char stacklevels[2*d * (tree_h + 1)*n];
  unsigned char auth[2*d * tree_h*n];
  unsigned char keep[2*d * (tree_h >> 1)*n];
  treehash_inst treehash[2*d * (tree_h-k)];
  unsigned char th_nodes[2*d * (tree_h-k)*n];
  unsigned char retain[2*d * ((1 << k) - k - 1)*n];
  unsigned char wots_sigs[d * params->xmss_par.wots_par.keysize];
  bds_state states[2*d];

  int i, j;

  for (i = 0; i < 2*d; i++) {
    for(j=0;j<h-k;j++) {
      treehash[i*(tree_h-k) + j].node = th_nodes + (i*(tree_h-k) + j) * n;
    }
    xmss_set_bds_state(states + i,
      stack + i*(tree_h + 1)*n, 0, stacklevels + i*(tree_h + 1),
      auth + i*tree_h*n,
      keep + i*(tree_h >> 1)*n,
      treehash + i*(tree_h-k),
      retain + i*((1 << k) - k - 1)*n,
      0
    );
  }

  // round (params->index_len+2*n+m) to nearest multiple of 4, then divide by 4
  int sk_length_words = (((params->index_len+2*n+m) + 4 - 1) & ~(4 - 1)) >> 2;

  unsigned char sk[4*sk_length_words];
  unsigned char pk[2*n];

  unsigned long long oldcount, newcount;

  oldcount = *DWT_CYCCNT;
  xmssmt_keypair(pk, sk, states, wots_sigs, params);
  newcount = (*DWT_CYCCNT)-oldcount;
  sprintf(output, "Cost: %d", newcount);
  SendUARTStr(output);

  sprintf(output, "Starting signing: %d signatures, mlen: %d", SIGNATURES, MLEN);
  SendUARTStr(output);

  for (i = 0; i < MLEN; i++)
    mi[i] = i;

  for (i = 0; i < SIGNATURES; i++) {
    oldcount = *DWT_CYCCNT;
    xmssmt_sign(sk, states, wots_sigs, mi, MLEN, params);
    newcount = (*DWT_CYCCNT)-oldcount;

    sprintf(output, "Cost: %d", newcount);
    SendUARTStr(output);
  }

  return 0;
}
