/*
version 20151006
Joost Rijneveld
Public domain.
*/

#include "stm32wrapper.h"
#include "params.h"
#include "xmss_fast.h"
#include <stdio.h>
#include <stdint.h>

#define MLEN 32

int main(void)
{
  clock_setup();
  gpio_setup();
  usart_setup();
  dma_request_setup();
  dma_transmit_setup();

  volatile unsigned int *DWT_CYCCNT = (unsigned int *)0xE0001004; //address of the register
  volatile unsigned int *DWT_CONTROL = (unsigned int *)0xE0001000; //address of the register
  volatile unsigned int *SCB_DEMCR = (unsigned int *)0xE000EDFC; //address of the register

  *SCB_DEMCR = *SCB_DEMCR | 0x01000000;
  *DWT_CYCCNT = 0; // reset the counter
  *DWT_CONTROL = *DWT_CONTROL | 1 ; // enable the counter

  char output[64];

  int m = XMSS_M;
  int n = XMSS_N;
  int h = XMSS_TREE_HEIGHT;
  int d = XMSSMT_LAYERS;
  int w = WOTS_W;
  int k = BDS_K;

  xmssmt_params p;
  xmssmt_params *params = &p;
  xmssmt_set_params(params, m, n, h, d, w, k);

  unsigned long long signature_length = params->index_len + m + (d*params->xmss_par.wots_par.keysize) + h*n;

  unsigned char mo[MLEN];
  unsigned char sm[MLEN+signature_length];
  unsigned long long mlen;

  unsigned char pk[2*n];

  unsigned long long oldcount, newcount;

  sprintf(output, "Starting verification");
  SendUARTStr(output);

  oldcount = *DWT_CYCCNT;
  // just using mock data for benchmark
  xmssmt_sign_open(mo, &mlen, sm, MLEN + signature_length, pk, params);
  newcount = (*DWT_CYCCNT)-oldcount;

  sprintf(output, "Cost: %d", newcount);
  SendUARTStr(output);

  return 0;
}
